/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: SubnetControl.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 * 
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public partial class SubnetControl : UserControl
    {
        private SubnetPropertyPage parentPage; 
        public SubnetControl()
        {
            InitializeComponent();
            tbName.Focus();
            parentPage = null;
        }

        public SubnetControl(SubnetPropertyPage parentPage)
        {
            InitializeComponent();
            tbName.Focus();
            this.parentPage = parentPage;
            tbMask.ReadOnly = true;
        }

        public Boolean GetData(out DhcpSubnet network, out DhcpIpRange range, out DhcpOptionValue value51)
        {
            if (!this.MyValidate())
            {
                network = null;
                range = null;
                value51 = null;
                return false;
            }
            network = new DhcpSubnet();

            range = new DhcpIpRange();
            range.Type = DhcpSubnetElementDataType.DhcpIpRangesDhcpOnly;
            range.StartAddress = new DhcpIpAddress(this.tbStartIp.Text.Trim());
            range.EndAddress = new DhcpIpAddress(this.tbEndIp.Text.Trim());

            network.Mask = new DhcpIpAddress(this.tbMask.Text.Trim());

            network.Address = new DhcpIpAddress(range.StartAddress.GetUIntAddress() & network.Mask.GetUIntAddress());
            network.Name = this.tbName.Text.Trim();
            network.Comment = this.tbDesc.Text.Trim();

            if (this.rbActive.Checked)
                network.State = DhcpSubnetState.Enabled;
            else
                network.State = DhcpSubnetState.Disabled;

            TimeSpan ts = new TimeSpan((int)this.nudDays.Value, (int)this.nudHours.Value,
                    (int)this.nudMinutes.Value, 0);

            value51 = new DhcpOptionValue();
            value51.OptionId = 51;
            value51.ClassType = DhcpOptionClassType.Dhcp;
            DhcpOptionDataDword od = new DhcpOptionDataDword();
            od.Type = DhcpOptionDataType.DWordType;
            od.Data = new UInt32[] { (UInt32)ts.TotalSeconds };
            value51.OptionData = od;

            DhcpOptionScope os = new DhcpOptionScope();
            os.type = DhcpOptionScopeType.Subnet;
            os.SubnetIp = network.Address;
            value51.OptionScope = os;

            return true;
        }

        public void SetData(DhcpSubnet network, DhcpIpRange range, DhcpOptionValue value51)
        {
            this.tbName.Text = network.Name ?? String.Empty;
            this.tbDesc.Text = network.Comment ?? String.Empty;

            if (range != null)
            {
                this.tbStartIp.Text = range.StartAddress.ToString();
                this.tbEndIp.Text = range.EndAddress.ToString();
            }
            else
            {
                this.tbStartIp.Text = this.tbEndIp.Text = "0.0.0.0";
            }
            this.tbMask.Text = network.Mask.ToString();

            if (network.State == DhcpSubnetState.Enabled ||
                network.State == DhcpSubnetState.EnabledSwitched)
                this.rbActive.Checked = true;
            else
                this.rbInactive.Checked = true;

            if (value51 != null)
            {

                TimeSpan ts = TimeSpan.FromSeconds(((DhcpOptionDataDword)value51.OptionData).Data[0]);

                this.nudDays.Value = ts.Days;
                this.nudHours.Value = ts.Hours;
                this.nudMinutes.Value = ts.Minutes;
            }
            else
                this.nudDays.Value = this.nudHours.Value = this.nudMinutes.Value = 0;

            if (this.parentPage != null)
            {
                this.parentPage.Dirty = this.parentPage.SubnetChanged = this.parentPage.RangeChanged = this.parentPage.LeaseTimeChanged = false;
            }
        }

        private void SubnetChanged(object sender, EventArgs e)
        {
            //if control is being used by propertey page
            if (this.parentPage != null)
            {
                this.parentPage.Dirty = this.parentPage.SubnetChanged  = true;
            }
        }

        private void RangeChanged(object sender, EventArgs e)
        {
            //if control is being used by propertey page
            if (this.parentPage != null)
            {
                this.parentPage.Dirty = this.parentPage.RangeChanged = true;
            }
        }

        private void LeaseTimeChanged(object sender, EventArgs e)
        {
            //if control is being used by propertey page
            if (this.parentPage != null)
            {
                this.parentPage.Dirty = this.parentPage.LeaseTimeChanged = true;
            }
        }

        public Boolean MyValidate()
        {
            String caption = "Input Error";
            //subnet name
            if (tbName.Text.Trim().Length < 1)
            {
                MessageBox.Show(this.ParentForm, "Name field blank", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //subnet start ip
            DhcpIpAddress startIp;
            try
            {
                startIp = new DhcpIpAddress(tbStartIp.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this.ParentForm, "Start IP not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            DhcpIpAddress endIp;
            try
            {
                endIp = new DhcpIpAddress(tbEndIp.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this.ParentForm, "End IP not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            DhcpIpAddress maskIp;
            try
            {
                maskIp = new DhcpIpAddress(tbMask.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this.ParentForm, "Mask not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //check mask
            if ((startIp.GetUIntAddress() & maskIp.GetUIntAddress())
                != (endIp.GetUIntAddress() & maskIp.GetUIntAddress()))
            {
                MessageBox.Show(this.ParentForm, "IP Range and Mask not correct", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //check sip < eip
            if (! (startIp.GetUIntAddress() < endIp.GetUIntAddress()) )
            {
                MessageBox.Show(this.ParentForm, "IP Range not correct", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //if sip equal network
            if (startIp.GetUIntAddress() == (startIp.GetUIntAddress() & maskIp.GetUIntAddress()))
            {
                MessageBox.Show(this.ParentForm, "Start IP cannot equal Network Address", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }
    }
}
