/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: ProfileScopeNode.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Windows.Forms;
using System.Security;
using Microsoft.ManagementConsole;
using Microsoft.ManagementConsole.Advanced;

using Unf.Dhcp.ManagementConsole.DwsOps;
using Unf.Dhcp.ManagementConsole.DwsSec;

namespace Unf.Dhcp.ManagementConsole 
{
    /// <summary>
    /// 
    /// </summary>
    public class ProfileScopeNode : ScopeNode
    {
        public ProfileSaveData SaveData;
        public DhcpOperations DhcpOps;
        public DhcpSecurity DhcpSec;
        public Boolean isAdmin = false;
        public ProfileScopeNode(ProfileSaveData data)
        {
            if (data == null)
                throw new ArgumentNullException();
            //save data
            this.SaveData = data;
            this.DisplayName = data.ProfileName;
            this.ImageIndex = this.SelectedImageIndex = 0;

            //verbs
            this.EnabledStandardVerbs = StandardVerbs.Properties | StandardVerbs.Delete | StandardVerbs.Rename | StandardVerbs.Refresh;

        }

        protected override void OnSyncAction(SyncAction action, SyncStatus status)
        {
            switch ((string)action.Tag)
            {
                case "AddServer":
                    {
                        ServerConnectForm connForm = new ServerConnectForm(this);

                        if (this.SnapIn.Console.ShowDialog(connForm) == DialogResult.OK)
                        {
                            this.SaveData.ServerIPs.Add(connForm.tbServerIp.Text.Trim());

                            this.Children.Insert(0, new ServerScopeNode(this, connForm.tbServerIp.Text.Trim()));

                            this.SnapIn.IsModified = true;
                        }
                        
                        break;
                    }
            }
        } 

        protected override void OnRename(string newText, SyncStatus status)
        {
            if (String.IsNullOrEmpty(newText.Trim()))
                return;

            this.DisplayName = this.SaveData.ProfileName = newText.Trim();
            this.SnapIn.IsModified = true;
        }

        protected override void OnDelete(SyncStatus status)
        {
            String msgText = "'" + this.DisplayName + "' Profile will be deleted permanently";
            if (SnapInUtil.PopUpConfirm(msgText) == DialogResult.OK)
            {
                this.Parent.Children.Remove(this);
                this.SnapIn.IsModified = true;
            }
        }

        protected override void OnRefresh(AsyncStatus status)
        {
            if (InitWebServices())
            {
                setData();

            }
        }

        protected override void OnExpand(AsyncStatus status)
        {
            if (InitWebServices())
            {
                setData();
            
            }
        }

        private void setData()
        {
            this.Children.Clear();
            this.ActionsPaneItems.Clear();

            //actions
            this.ActionsPaneItems.Add(new SyncAction("Manage DHCP Server...", "Add a DHCP Server", 6, "AddServer"));

            //add saved servers
            foreach (string server in this.SaveData.ServerIPs)
                this.Children.Add(new ServerScopeNode(this, server));
            
            if(this.isAdmin)
                this.Children.Add(new PermissionsScopeNode(this, null));
        }

        protected override void OnAddPropertyPages(PropertyPageCollection propertyPageCollection)
        {
            propertyPageCollection.Add(new ProfilePropertyPage(this));
        }

        private Boolean InitWebServices()
        {
            if (this.DhcpOps == null)
            {   
                PasswordForm passform = new PasswordForm();
                passform.lblUserName.Text = this.SaveData.UserName;
                passform.lblDomainName.Text = this.SaveData.LoginDomain;
                passform.lblProfileName.Text = this.SaveData.ProfileName;
                if (this.SnapIn.Console.ShowDialog(passform) == DialogResult.OK)
                {
                    try
                    {
                        //create dws objects
                        this.DhcpOps = new DhcpOperations();
                        this.DhcpSec = new DhcpSecurity();
                        this.DhcpOps.Url = this.SaveData.DwsUrl + "/DhcpOperations.asmx";
                        this.DhcpSec.Url = this.SaveData.DwsUrl + "/DhcpSecurity.asmx";

                        //create creds
                        CredentialCache cc = new CredentialCache();
                        NetworkCredential cred = new NetworkCredential(this.SaveData.UserName, passform.tbPassword.Text, this.SaveData.LoginDomain);
                        passform.tbPassword.Clear();

                        cc.Add(new Uri(this.DhcpOps.Url), "Basic", cred);
                        cc.Add(new Uri(this.DhcpSec.Url), "Basic", cred);

                        //assign creds
                        this.DhcpOps.Credentials = cred;
                        this.DhcpSec.Credentials = cred;


                        this.isAdmin = this.DhcpSec.isAdmin();
                            

                    }
                    catch (Exception e)
                    {
                        SnapInUtil.PopUpError("Profile Error",
                            "Unabled to Connect to Dhcp Web Services at: \r\n"
                            + this.SaveData.DwsUrl + Environment.NewLine
                            + "Error: " + e.Message);
                        this.DhcpOps = null;
                        return false;
                    }
                    return true;
                }
                else
                    return false;
            }
            return true;
        }
    }

    [Serializable]
    public class ProfileSaveData
    {
        public String ProfileName = String.Empty;
        public String DwsUrl = String.Empty;
        public String UserName = String.Empty;
        public String LoginDomain = String.Empty;
        public List<String> ServerIPs = new List<string>(0);

        public ProfileSaveData(){}
    }
}
