/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: SubnetScopeNode.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;


namespace Unf.Dhcp.ManagementConsole
{
    public class SubnetScopeNode : ScopeNode
    {
        public ProfileScopeNode ProfileNode;
        public ServerScopeNode ServerNode;
        public DhcpSubnet DhcpSubnet;
        public DhcpIpAddress ServerIp;

        public SubnetScopeNode(ServerScopeNode serverNode, DhcpSubnet subnet)
        {
            this.DhcpSubnet = subnet;
            this.ServerIp = serverNode.ServerIp;
            this.ProfileNode = serverNode.ProfileNode;
            this.ServerNode = serverNode;
        
            this.DisplayName = "Network " + subnet.Address.Ip;
            this.ImageIndex = this.SelectedImageIndex = 6;
            //Default List View additions
            this.SubItemDisplayNames.Add(this.DhcpSubnet.Mask.Ip ?? String.Empty);
            this.SubItemDisplayNames.Add(this.DhcpSubnet.Name ?? String.Empty);
            this.SubItemDisplayNames.Add(this.DhcpSubnet.State.ToString());
            this.SubItemDisplayNames.Add(this.DhcpSubnet.Comment ?? String.Empty);
 
            //view testing
            AddViewDescription("Default View", typeof(NormalListView));
            ViewDescriptions.DefaultIndex = 0;
        
        }

        private void SetData()
        {
            
            //add pool
            this.Children.Add(new PoolScopeNode(this));

            //add leases
            this.Children.Add(new LeaseScopeNode(this));

            //add rezs
            this.Children.Add(new RezScopeNode(this));

            //add option
            this.Children.Add(new OptionScopeNode((ServerScopeNode)this.Parent, new DhcpOptionScope(this.DhcpSubnet.Address)));

            //add permission node
            if (ProfileNode.isAdmin)
            {

                DwsSec.DhcpAzSubnetScope azScope = new DwsSec.DhcpAzSubnetScope();
                azScope.Server = new DwsSec.DhcpIpAddress(ServerIp.Ip);
                azScope.Subnet = new DwsSec.DhcpIpAddress(DhcpSubnet.Address.Ip);


                this.Children.Add(new PermissionsScopeNode(ProfileNode, azScope));
                this.Children.Add(new IprPermissionsScopeNode(this));

            }


            //verbs
            this.EnabledStandardVerbs = StandardVerbs.Refresh | StandardVerbs.Delete | StandardVerbs.Properties;
        }

        protected override void OnExpand(AsyncStatus status)
        {
            this.SetData();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnDelete(SyncStatus status)
        {
            try
            {
                String msgText = "'" + this.DisplayName + "' Subnet will be deleted permanently";
                if (SnapInUtil.PopUpConfirm(msgText) == DialogResult.OK)
                {
                    ProfileNode.DhcpOps.DeleteSubnet(ServerIp, DhcpSubnet.Address);
                    ((ServerScopeNode)this.Parent).Children.Remove(this);
                }
            }
            catch (Exception e)
            {
                SnapInUtil.PopUpError("Subnet Delete Error",
                                "Unabled to Delete Subnet" + Environment.NewLine
                                + "Error: " + e.Message);
            } 
        }

        protected override void OnAddPropertyPages(PropertyPageCollection propertyPageCollection)
        {
            propertyPageCollection.Add(new SubnetPropertyPage(this));
            propertyPageCollection.Add(new DnsPropertyPage(this));
            propertyPageCollection.Add(new SubnetAdvPropertyPage(this));
        }

        #region Switch Views...
        /// <summary>
        /// Adds a view description and set it as the default
        /// </summary>
        /// <param name="displayName"></param>
        /// <param name="viewType"></param>
        public void AddViewDescription(string displayName, Type viewType)
        {
            // add the view description (display name is required)
            //List View
            MmcListViewDescription lvd = new MmcListViewDescription();
            lvd.DisplayName = displayName;
            lvd.ViewType = viewType;
            if(viewType == typeof(SearchListView))
                lvd.Options = MmcListViewOptions.ExcludeScopeNodes | MmcListViewOptions.HideSelection;
            this.ViewDescriptions.Add(lvd);

        }

        /// <summary>
        /// Removes a message view description 
        /// </summary>
        /// <param name="removeIndex"></param>
        public void RemoveViewDescriptionAt(int removeIndex)
        {
            // remove the description
            this.ViewDescriptions.RemoveAt(removeIndex);
        }

        /// <summary>
        /// Swaps a new view for the existing. 
        /// (NOTE: expects that only a single view is already defined)
        /// </summary>
        /// <param name="currentView"></param>
        /// <param name="newViewDisplayName"></param>
        /// <param name="newViewType"></param>
        public void SwapView(Microsoft.ManagementConsole.View currentView, string newViewDisplayName, Type newViewType)
        {
            // remove the existing view description 
            this.RemoveViewDescriptionAt(0);

            // add the new view description
            this.AddViewDescription(newViewDisplayName, newViewType);

            // set the new view as the default
            this.ViewDescriptions.DefaultIndex = 0;

            // trigger MMC to show the new default view 
            currentView.SelectScopeNode(this);
        }
        #endregion
    }


    
}
