/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: IprPermissionListView.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsSec;


namespace Unf.Dhcp.ManagementConsole
{
    class IprPermissionListView : MmcListView
    {
        protected override void OnInitialize(AsyncStatus status)
        {
            base.OnInitialize(status);

            // Create a set of columns for use in the list view
            // Define the default column title
            this.Columns[0].Title = "Start IP";
            this.Columns[0].SetWidth(100);


            // Add detail column
            this.Columns.Add(new MmcListViewColumn("End IP", 100));
            this.Columns.Add(new MmcListViewColumn("Account", 100));
            this.Columns.Add(new MmcListViewColumn("Role", 100));

            //listen for scope event
            ((IprPermissionsScopeNode)this.ScopeNode).Changed += new IprPermissionsScopeNode.ChangedDelegate(OnScopeNodeChange);

            //allow selected node to be deleted
            this.SelectionData.EnabledStandardVerbs = StandardVerbs.Delete;

            Refresh();
        }

        /// <summary>
        /// Cleanup. 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnShutdown(SyncStatus status)
        {
            //Stop listening for scope event
            ((IprPermissionsScopeNode)this.ScopeNode).Changed -= new IprPermissionsScopeNode.ChangedDelegate(OnScopeNodeChange);
        }

        protected void Refresh()
        {
            this.ResultNodes.Clear();

            SubnetScopeNode subnetNode = ((IprPermissionsScopeNode)this.ScopeNode).SubnetNode;

            DhcpAzSubnetScope azScope = new DhcpAzSubnetScope();
            azScope.ScopeType = DhcpAzScopeType.Subnet;
            azScope.Server = new DhcpIpAddress(subnetNode.ServerIp.ToString()); ;
            azScope.Subnet = new DhcpIpAddress(subnetNode.DhcpSubnet.Address.ToString());
            DhcpAzIpRangeScope[] ranges = new DhcpAzIpRangeScope[0];
            try
            {
                ranges = subnetNode.ProfileNode.DhcpSec.EnumAzIpRanges(azScope);
                if (ranges == null)
                    return;
                foreach (DhcpAzIpRangeScope range in ranges)
                {
                    DhcpAzAccount[] accounts = subnetNode.ProfileNode.DhcpSec.GetPermissions(range);
                    if (accounts == null)
                        return;
                    foreach (DhcpAzAccount account in accounts)
                    {
                        ResultNode r = new ResultNode();
                        r.ImageIndex = 1;
                        r.DisplayName = range.StartIp.ToString();
                        r.SubItemDisplayNames.Add(range.EndIp.ToString());
                        r.SubItemDisplayNames.Add(account.AccountName);
                        r.SubItemDisplayNames.Add(account.Role.ToString());
                        this.ResultNodes.Add(r);
                    }
                }
            }
            catch
            {
                throw;
            }

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnSelectionChanged(SyncStatus status)
        {
            if (this.SelectedNodes.Count == 0)
            {
                this.SelectionData.Clear();
            }
            else
            {
                this.SelectionData.DisplayName = this.SelectedNodes[0].SubItemDisplayNames[1];
                this.SelectionData.Update(this.SelectedNodes[0], false, null, null);
                this.SelectionData.ActionsPaneItems.Clear();
            }
        }

        protected override void OnDelete(SyncStatus status)
        {
            try
            {
                DhcpAzAccount account = new DhcpAzAccount();
                account.AccountName = this.SelectedNodes[0].SubItemDisplayNames[1];
                account.Role = (DhcpAzRole)Enum.Parse(typeof(DhcpAzRole), this.SelectedNodes[0].SubItemDisplayNames[2]);
                SubnetScopeNode node = ((IprPermissionsScopeNode)this.ScopeNode).SubnetNode;
                DhcpAzIpRangeScope azScope = new DhcpAzIpRangeScope();
                azScope.ScopeType = DhcpAzScopeType.IpRange;
                azScope.Server = new DhcpIpAddress(node.ServerIp.ToString());
                azScope.Subnet = new DhcpIpAddress(node.DhcpSubnet.Address.ToString());
                azScope.StartIp = new DhcpIpAddress(this.SelectedNodes[0].DisplayName);
                azScope.EndIp = new DhcpIpAddress(this.SelectedNodes[0].SubItemDisplayNames[0]);

                node.ProfileNode.DhcpSec.RemovePermission(azScope, account);

                this.ResultNodes.Remove((ResultNode)this.SelectedNodes[0]);
            }
            catch (Exception e)
            {
                SnapInUtil.PopUpError("Permission Delete Error",
                            "Unabled to Delete Permission" + Environment.NewLine
                            + "Error: " + e.Message);
            }
        }

        /// <summary>
        /// Handle any change to the scope node. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OnScopeNodeChange(object sender, EventArgs ev)
        {
            this.Refresh();
        }
    }
}
