/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: PoolListView.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    class PoolListView : MmcListView
    {
        public PoolListView() { }

        protected override void OnInitialize(AsyncStatus status)
        {
            base.OnInitialize(status);

            // Create a set of columns for use in the list view
            // Define the default column title
            this.Columns[0].Title = "Pool Type";
            this.Columns[0].SetWidth(100);

            // Add detail column
            this.Columns.Add(new MmcListViewColumn("Start Ip", 125));
            this.Columns.Add(new MmcListViewColumn("End Ip", 125));
            this.Columns.Add(new MmcListViewColumn("Description", 200));

            //listen for scope event
            ((PoolScopeNode)this.ScopeNode).Changed += new PoolScopeNode.ChangedDelegate(OnScopeNodeChange);


            this.Refresh();
        }

        /// <summary>
        /// Cleanup. 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnShutdown(SyncStatus status)
        {
            //Stop listening for scope event
            ((PoolScopeNode)this.ScopeNode).Changed -= new PoolScopeNode.ChangedDelegate(OnScopeNodeChange);
        }

        protected override void OnDelete(SyncStatus status)
        {
            PoolScopeNode parent = (PoolScopeNode)this.ScopeNode;
            DhcpIpRange exRange = new DhcpIpRange();
            exRange.Type = DhcpSubnetElementDataType.DhcpExcludedIpRanges;
            exRange.StartAddress = new DhcpIpAddress(this.SelectedNodes[0].SubItemDisplayNames[0]);
            exRange.EndAddress = new DhcpIpAddress(this.SelectedNodes[0].SubItemDisplayNames[1]);

            try
            {
                parent.SubnetNode.ProfileNode.DhcpOps.DeleteSubnetExclusion(parent.SubnetNode.ServerIp,
                    parent.SubnetNode.DhcpSubnet.Address, exRange);
                this.ResultNodes.Remove((ResultNode)this.SelectedNodes[0]);

                parent.Pools.Remove(exRange);
            }
            catch (Exception e)
            {
                SnapInUtil.PopUpError("Pool Error", "Unable to delete DHCP Pool" + Environment.NewLine + "Error: " + e.Message);
            }
        }

        private void Refresh()
        {
            PoolScopeNode parent = (PoolScopeNode)this.ScopeNode;
            
            this.ResultNodes.Clear();

            foreach (DhcpIpRange pool in parent.Pools)
            {
                ResultNode r = new ResultNode();

                r.DisplayName = (String)RangeTypeMap.ht[pool.Type];
                r.ImageIndex = 6;
                r.SubItemDisplayNames.Add(pool.StartAddress.ToString() ?? String.Empty);
                r.SubItemDisplayNames.Add(pool.EndAddress.ToString() ?? String.Empty);

                if (pool.Type == DhcpSubnetElementDataType.DhcpExcludedIpRanges)
                    r.SubItemDisplayNames.Add("IP range excluded from leasing");
                else
                    r.SubItemDisplayNames.Add("Subnet IP range for leasing");

                this.ResultNodes.Add(r);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnSelectionChanged(SyncStatus status)
        {
            if (this.SelectedNodes.Count == 0)
            {
                this.SelectionData.Clear();
            }
            else
            {
                this.SelectionData.DisplayName = this.SelectedNodes[0].SubItemDisplayNames[0] + " - " + this.SelectedNodes[0].SubItemDisplayNames[1];
                this.SelectionData.Update(this.SelectedNodes[0], false, null, null);
                this.SelectionData.ActionsPaneItems.Clear();

                //allow selected node to be deleted
                if (this.SelectedNodes[0].DisplayName == "Exclusion")    
                    this.SelectionData.EnabledStandardVerbs = StandardVerbs.Delete;
                else
                    this.SelectionData.EnabledStandardVerbs = StandardVerbs.None;
            }
        }

        /// <summary>
        /// Handle any change to the scope node. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OnScopeNodeChange(object sender, EventArgs ev)
        {
            this.Refresh();
        }

    }

    static internal class RangeTypeMap
    {
        public static Hashtable ht = new Hashtable();
        static RangeTypeMap()
        {
            ht.Add(DhcpSubnetElementDataType.DhcpExcludedIpRanges, "Exclusion");
            ht.Add(DhcpSubnetElementDataType.DhcpIpRangesDhcpOnly, "Dhcp");
            ht.Add(DhcpSubnetElementDataType.DhcpIpRangesBootpOnly, "Bootp");
            ht.Add(DhcpSubnetElementDataType.DhcpIpRangesDhcpBootp, "DhcpBootp");
            
        }
    }
}
