/* 
 * DHCP Server Management Objects
 *
 *  File: DhcpAzScope.cs
 *  Namespace: Unf.Dhcp.Smo.WebServices
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Text.RegularExpressions;
using Microsoft.Interop.Security.AzRoles;

namespace Unf.Dhcp.Smo.WebServices
{
    /// <summary>
    /// Base class for DHCP authorization scopes
    /// </summary>
    public abstract class DhcpAzScope
    {
        /// <summary>
        /// Scope type
        /// </summary>
        public DhcpAzScopeType ScopeType;
        /// <summary>
        /// Dummy constructor
        /// </summary>
        public DhcpAzScope() { }

        /// <summary>
        /// Generates the scope string ID based on the scopes type and members
        /// </summary>
        /// <returns>A string representing the scope's ID in the AzMan database</returns>
        public abstract String GenerateAzScopeName() ;

        /// <summary>
        /// A scope specific check access authorization
        /// </summary>
        /// <param name="opDesc">Audit description</param>
        /// <param name="azClient">Client context (current httpcontext user)</param>
        /// <param name="operation">Authorized operation</param>
        /// <returns>True, if client is allowed to perform operation in scope</returns>
        public abstract Boolean AccessCheck(String opDesc, IAzClientContext2 azClient, DhcpAzOps operation);
    }

    /// <summary>
    /// DHCP server authorization scope (server level roles/permissions)
    /// </summary>
    public class DhcpAzServerScope : DhcpAzScope
    {
        /// <summary>
        /// DHCP server IP address
        /// </summary>
        public DhcpIpAddress Server;
        /// <summary>
        /// Dummy constructor, sets type
        /// </summary>
        public DhcpAzServerScope() { this.ScopeType = DhcpAzScopeType.Server; }

        /// <summary>
        /// Generates the scope string ID based on DHCP server's IP
        /// </summary>
        /// <returns>A string representing the scope's ID in the AzMan database</returns>
        /// <remarks>String ID format: 'SRV w.x.y.z'</remarks>
        public override string GenerateAzScopeName()
        {
            if (this.Server == null)
                throw new InvalidOperationException();
            return "SRV " + this.Server.ToString();
        }

        /// <summary>
        /// Generates the scope string ID based on server parameter
        /// </summary>
        /// <returns>A string representing the scope's ID in the AzMan database</returns>
        /// <remarks>String ID format: 'SRV w.x.y.z'</remarks>
        protected static string GenerateAzScopeName(DhcpIpAddress server)
        {
            return "SRV " + server.ToString();
        }

        /// <summary>
        /// A server scope check access authorization
        /// </summary>
        /// <param name="opDesc">Audit description</param>
        /// <param name="azClient">Client context (current httpcontext user)</param>
        /// <param name="operation">Authorized operation</param>
        /// <returns>True, if client is allowed to perform operation at server level</returns>
        public override Boolean AccessCheck(String opDesc, IAzClientContext2 azClient, DhcpAzOps operation)
        {
            Object[] result;
            Object[] azOp = new Object[] { operation };
            String[] scopes = new String[] { "", this.GenerateAzScopeName() };

            foreach (String scope in scopes)
            {
                try
                {
                    result = (Object[])azClient.AccessCheck(opDesc, new Object[] { scope }, azOp, null, null, null, null, null);
                    if ((int)result[0] == 0)
                        return true;
                }
                catch { }
            }

            return false;
        }
    }

    /// <summary>
    /// DHCP subnet authorization scope (subnet level roles/permissions)
    /// </summary>
    public class DhcpAzSubnetScope : DhcpAzServerScope
    {
        /// <summary>
        /// DHCP subnet network IP address
        /// </summary>
        public DhcpIpAddress Subnet;
        /// <summary>
        /// Dummy construtor, sets type
        /// </summary>
        public DhcpAzSubnetScope() { this.ScopeType = DhcpAzScopeType.Subnet; }

        /// <summary>
        /// Generates the scope string ID based on inherited server IP and this subnet IP
        /// </summary>
        /// <returns>A string representing the scope's ID in the AzMan database</returns>
        /// <remarks>String ID format: 'SRV w.x.y.z - SN a.b.c.d'</remarks>
        public override string GenerateAzScopeName()
        {
            if (this.Server == null || this.Subnet == null)
                throw new InvalidOperationException();
            return "SRV " + this.Server.ToString() + " - SN " + this.Subnet.ToString();
        }

        /// <summary>
        /// Generates the scope string ID based on server IP and subnet IP parameters
        /// </summary>
        /// <returns>A string representing the scope's ID in the AzMan database</returns>
        /// <remarks>String ID format: 'SRV w.x.y.z - SN a.b.c.d'</remarks>
        protected static string GenerateAzScopeName(DhcpIpAddress server, DhcpIpAddress subnet)
        {
            return "SRV " +server.ToString() + " - SN " + subnet.ToString();
        }

        /// <summary>
        /// A subnet scope check access authorization
        /// </summary>
        /// <param name="opDesc">Audit description</param>
        /// <param name="azClient">Client context (current httpcontext user)</param>
        /// <param name="operation">Authorized operation</param>
        /// <returns>True, if client is allowed to perform operation at subnet level</returns>
        public override Boolean AccessCheck(String opDesc, IAzClientContext2 azClient, DhcpAzOps operation)
        {
            Object[] result;
            Object[] azOp = new Object[] { operation };
            String[] scopes = new String[] { "", DhcpAzServerScope.GenerateAzScopeName(this.Server), this.GenerateAzScopeName() };
            
            foreach(String scope in scopes)
            {
                try
                {
                    result = (Object[])azClient.AccessCheck(opDesc, new Object[] { scope }, azOp, null, null, null, null, null);
                    if ((int)result[0] == 0)
                        return true;
                }
                catch { }
            }

            return false;
        }
    }

    /// <summary>
    /// DHCP IP range authorization scope (IP range level roles/permissions)
    /// </summary>
    public class DhcpAzIpRangeScope : DhcpAzSubnetScope
    {
        /// <summary>
        /// Start IP address
        /// </summary>
        public DhcpIpAddress StartIp;
        /// <summary>
        /// End IP address
        /// </summary>
        public DhcpIpAddress EndIp;
        /// <summary>
        /// Dummy constructor, sets type
        /// </summary>
        public DhcpAzIpRangeScope() { this.ScopeType = DhcpAzScopeType.IpRange; }

        /// <summary>
        /// Generates the scope string ID based on IP range in inherited server and subnet IPs
        /// </summary>
        /// <returns>A string representing the scope's ID in the AzMan database</returns>
        /// <remarks>String ID format: 'SRV w.x.y.z - SN a.b.c.d - IPR e.f.g.h:i.j.k.l'<br/>
        /// StartIP can equal EndIP, creating a one address authorization
        /// </remarks>
        public override string GenerateAzScopeName()
        {
            if (this.Server == null || this.Subnet == null || this.StartIp == null || this.EndIp == null)
                throw new InvalidOperationException();
            return "SRV " + this.Server.ToString() + " - SN " + this.Subnet.ToString() +
                " - IPR " + this.StartIp + ":" + this.EndIp;
        }

        /// <summary>
        /// A IP range scope check access authorization
        /// </summary>
        /// <param name="opDesc">Audit description</param>
        /// <param name="azClient">Client context (current httpcontext user)</param>
        /// <param name="operation">Authorized operation</param>
        /// <returns>True, if client is allowed to perform operation at IP range level</returns>
        public override Boolean AccessCheck(String opDesc, IAzClientContext2 azClient, DhcpAzOps operation)
        {
            Object[] result;
            Object[] azOp = new Object[] { operation };
            String[] scopes = new String[] { "", DhcpAzServerScope.GenerateAzScopeName(this.Server), 
                DhcpAzSubnetScope.GenerateAzScopeName(this.Server, this.Subnet) };//, this.GenerateAzScopeName() };

            //check scope hierarchy
            foreach (String scope in scopes)
            {
                try
                {
                    result = (Object[])azClient.AccessCheck(opDesc, new Object[] { scope }, azOp, null, null, null, null, null);
                    if ((int)result[0] == 0)
                        return true;
                }
                catch { }
            }

            //find all scope this user belongs to
            Object cursor = null;
            Object[] clientScopes = (Object[])azClient.GetAssignedScopesPage(0, 2000, ref cursor);
            Regex filter = new Regex(@"^" + scopes[2] + @"\s-\sIPR\s(\d.*):(\d.*)$");
            foreach (String clientScope in clientScopes)
            {
                Match m = filter.Match(clientScope);
                if (m.Success)
                {
                    DhcpIpAddress startIp = new DhcpIpAddress(m.Groups[1].Value);
                    DhcpIpAddress endIp = new DhcpIpAddress(m.Groups[2].Value.Trim());

                    //isbetween?  if so, check this scope to see if operation allowed
                    if (startIp.GetUIntAddress() <= this.StartIp.GetUIntAddress() &&
                     this.StartIp.GetUIntAddress() <= endIp.GetUIntAddress())
                    {
                        try
                        {
                            result = (Object[])azClient.AccessCheck(opDesc, new Object[] { clientScope }, azOp, null, null, null, null, null);
                            if ((int)result[0] == 0)
                                return true;
                        }
                        catch { }
                    }
                }
            }

            return false;
        }
    }

    /// <summary>
    /// DHCP authorization scope types
    /// </summary>
    public enum DhcpAzScopeType
    {
        /// <summary>
        /// IP range level authorizations
        /// </summary>
        IpRange = 0,
        /// <summary>
        /// Subnet level authorizations
        /// </summary>
        Subnet = 1,
        /// <summary>
        /// Server level authorizations
        /// </summary>
        Server = 2
    }
}
