/* 
 * DHCP Server Management Objects
 *
 *  File: DhcpOperations.asmx.cs
 *  Namespace: Unf.Dhcp.Smo.WebServices
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. 
 *
*/
using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Collections.Generic;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.ComponentModel;
using System.Xml.Serialization;


using Unf.Dhcp.Smo;
using Unf.Dhcp.Smo.Operations;

namespace Unf.Dhcp.Smo.WebServices
{
    /// <summary>
    /// DHCP Web Services
    /// </summary>
    [WebService(Namespace = "http://www.unf.edu/~jrupard/dws/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    public class DhcpOperations : System.Web.Services.WebService
    {

        #region Subnet Ops...
        /// <summary>
        /// Enumerate subnets on server
        /// </summary>
        /// <param name="server">Server IP address to enumerate</param>
        /// <permission>Requires internal limited role.  This role will be automatically assigned based on other roles within the subnet and/or IP range</permission>
        /// <remarks>This method implements security trimming.  User will only 'see' subnets they can access</remarks>
        /// <returns>An array of DHCP subnets</returns>
        [WebMethod(Description = "Enumerate subnets on DHCP server")]
        public DhcpSubnet[] EnumSubnets(DhcpIpAddress server)
        {
            return AzServerOperations.EnumSubnets(server);
        }
       
        //secured
        /// <summary>
        /// Create a DHCP subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address</param>
        /// <param name="ipRange">Default address pool associated with subnet</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role at server scope</permission>
        /// <remarks>ipRange parameter can be of type <see cref="DhcpIpRange"/> or <see cref="BootpIpRange"/></remarks>
        [WebMethod(Description = "Create a DHCP subnet on server")]
        [XmlInclude(typeof(DhcpIpRange)), XmlInclude(typeof(BootpIpRange))]
        public void CreateSubnet(DhcpIpAddress server, DhcpSubnet network, DhcpIpRange ipRange)
        {
            AzServerOperations.CreateSubnet(server, network, ipRange);
        }

        //secured
        /// <summary>
        /// Update DHCP subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet object to update</param>
        /// <remarks>Operation will update network.Name, network.Comment, and network.State on existing subnet matching network.Address</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        [WebMethod(Description = "Update DHCP subnet on server")]
        public void UpdateSubnet(DhcpIpAddress server, DhcpSubnet network)
        {
            AzServerOperations.UpdateSubnet(server, network);
        }

        //secured
        /// <summary>
        /// Update default DHCP subnet address pool (range)
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to update pool on</param>
        /// <param name="ipRange">New default address pool associated with subnet</param>
        /// <remarks>
        /// ipRange parameter can be of type <see cref="DhcpIpRange"/> or <see cref="BootpIpRange"/><br/>
        /// Removes current address pool and replaces with ipRange.  This can be used to change address pool client type (Dhcp, Bootp, or Both)
        /// </remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        [WebMethod(Description = "Update default DHCP subnet address pool (range)")]
        public void UpdateSubnetRange(DhcpIpAddress server, DhcpIpAddress network, DhcpIpRange ipRange)
        {
            AzServerOperations.UpdateSubnetRange(server, network, ipRange);
        }

        //secured
        /// <summary>
        /// Delete DHCP subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address identifying subnet</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role at server scope</permission>
        [WebMethod(Description = "Delete DHCP subnet on server")]
        public void DeleteSubnet(DhcpIpAddress server, DhcpIpAddress network)
        {
            AzServerOperations.DeleteSubnet(server, network);
        }

        //secured
        /// <summary>
        /// Create a subnet exclusion address pool
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to create pool in</param>
        /// <param name="ipExRange">IP range use to created pool</param>
        /// <remarks>ipExRange.Type must equal <see cref="DhcpSubnetElementDataType.DhcpExcludedIpRanges"/></remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        [WebMethod(Description = "Create a subnet exclusion address pool")]
        public void CreateSubnetExclusion(DhcpIpAddress server, DhcpIpAddress network, DhcpIpRange ipExRange)
        {
            AzServerOperations.CreateSubnetExclusion(server, network, ipExRange);
        }

        //secured
        /// <summary>
        /// Deletes a subnet exclusion address pool
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to delete pool from</param>
        /// <param name="ipExRange">IP range to delete</param>
        /// <remarks>ipExRange.Type must equal <see cref="DhcpSubnetElementDataType.DhcpExcludedIpRanges"/></remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        [WebMethod(Description = "Delete a subnet exclusion address pool")]
        public void DeleteSubnetExclusion(DhcpIpAddress server, DhcpIpAddress network, DhcpIpRange ipExRange)
        {
            AzServerOperations.DeleteSubnetExclusion(server, network, ipExRange);
        }

        /// <summary>
        /// Enumerate all address pools belonging to subnet
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address</param>
        /// <returns>An array of address pools.  The first pool will be the default address pool followed by zero or more exclusion pools</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role at subnet scope</permission>
        [WebMethod(Description = "Enumerate all address pools belonging to subnet")]
        public DhcpIpRange[] EnumPools(DhcpIpAddress server, DhcpIpAddress network)
        {
            return AzServerOperations.EnumPools(server, network);
        }

        //secured
        /// <summary>
        /// Search subnet for client leases matching terms
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <returns>An array of client leases matching terms</returns>
        /// <remarks>Client filter terms are .NET regex patterns<br/>
        /// This method implements security trimming.  User will only 'see' client leases their authorized to access
        /// </remarks>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning IP address of client lease</permission>
        [WebMethod(Description = "Search subnet for client leases matching terms")]
        public DhcpClient[] SubnetFindAllClients(DhcpIpAddress server, DhcpIpAddress network, DhcpClientFilter terms)
        {
            return AzServerOperations.SubnetFindAllClients(server, network, terms);
        }

        //secured
        /// <summary>
        /// Search subnet for first client lease matching terms
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <remarks>Client filter terms are .NET regex patterns<br/>
        /// This method implements security trimming.  User will only 'see' client leases their authorized to access
        /// </remarks>
        /// <returns>First matching client lease or null if no match</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning IP address of client lease</permission>
        [WebMethod(Description = "Search subnet for first client lease matching terms")]
        public DhcpClient SubnetFindOneClient(DhcpIpAddress server, DhcpIpAddress network, DhcpClientFilter terms)
        {
            return AzServerOperations.SubnetFindOneClient(server, network, terms);
        }

        //secured
        /// <summary>
        /// Search <b>all</b> subnets on server looking for client leases matching terms
        /// </summary>
        /// <param name="server">Server IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <remarks>Client filter terms are .NET regex patterns
        /// </remarks>
        /// <returns>An array of client leases matching terms</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role at server scope</permission>
        [WebMethod(Description = "Search <b>all</b> subnets on server looking for client leases matching terms")]
        public DhcpClient[] ServerFindAllClients(DhcpIpAddress server, DhcpClientFilter terms)
        {
            return AzServerOperations.ServerFindAllClients(server, terms);
        }

        //secured
        /// <summary>
        /// Search <b>all</b> subnets on server looking for first client lease matching terms
        /// </summary>
        /// <param name="server">Server IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <remarks>Client filter terms are .NET regex patterns
        /// </remarks>
        /// <returns>First matching client lease or null if no match</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role at server scope</permission>
        [WebMethod(Description = "Search <b>all</b> subnets on server looking for first client lease matching terms")]
        public DhcpClient ServerFindOneClient(DhcpIpAddress server, DhcpClientFilter terms)
        {
            return AzServerOperations.ServerFindOneClient(server, terms);
        }

#endregion

        #region Lease Ops...
        /// <summary>
        /// Enumerate all client leases on subnet
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to enumerate</param>
        /// <returns>A list of client leases belonging to subnet</returns>
        /// <remarks>This method implements security trimming.  User will only 'see' client leases their authorized to access</remarks>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning IP address of client lease</permission>
        [WebMethod(Description = "Enumerate all client leases on subnet")]
        public DhcpClient[] EnumLeases(DhcpIpAddress server, DhcpIpAddress network)
        {
            return AzServerOperations.EnumLeases(server, network);
        }

        /// <summary>
        /// Create a client lease on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="lease">Client lease info to create</param>
        /// <remarks>
        /// Client lease will be assigned to a existing subnet based on the binary AND operation of lease.IpAddress &amp; lease.SubnetMask
        /// </remarks>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning IP address of client lease</permission>
        [WebMethod(Description = "Create a client lease on server")]
        public void CreateLease(DhcpIpAddress server, DhcpClient lease)
        {
            AzServerOperations.CreateLease(server, lease);
        }

        /// <summary>
        /// Update a client lease on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="lease">Client lease to match and update</param>
        /// <remarks>
        /// Client lease will be updated on a existing subnet based on the binary AND operation of lease.IpAddress &amp; lease.SubnetMask<br/>
        /// Operation will update lease.Mac, lease.Name, lease.Comment, and lease.LeaseExpires on existing lease matching lease.IpAddress and lease.SubnetMask
        /// </remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role on scope owning IP address of client lease</permission>
        [WebMethod(Description = "Update a client lease on server")]
        public void UpdateLease(DhcpIpAddress server, DhcpClient lease)
        {
            AzServerOperations.UpdateLease(server, lease);
        }

        /// <summary>
        /// Delete client lease from server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to delete lease from</param>
        /// <param name="leaseIp">Client lease IP address, used to identify and delete</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning IP address of client lease</permission>
        [WebMethod(Description = "Delete client lease from server")]
        public void DeleteLease(DhcpIpAddress server, DhcpIpAddress network, DhcpIpAddress leaseIp)
        {
            AzServerOperations.DeleteLease(server, network, leaseIp);
        }
        #endregion

        #region Rez Ops...
        //requires ip+ create
        /// <summary>
        /// Create a DHCP IP reservation on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to create reservation on</param>
        /// <param name="res">IP reservation info</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning reservation's IP address</permission>
        [WebMethod(Description = "Create a DHCP IP reservation on server")]
        public void CreateReservation(DhcpIpAddress server, DhcpIpAddress network, DhcpReservation res)
        {
            AzServerOperations.CreateReservation(server, network, res);
        }

        /// <summary>
        /// Update a DHCP IP reservation on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to update reservation on</param>
        /// <param name="res">Updated IP reservation info</param>
        /// <remarks>Updates res.ReservedMac, res.bAllowedClientTypes, res.Name, and res.Comment based on match of res.ReservedIp</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role on scope owning reservation's IP address</permission>
        [WebMethod(Description = "Update a DHCP IP reservation on server")]
        public void UpdateReservation(DhcpIpAddress server, DhcpIpAddress network, DhcpReservation res)
        {
            AzServerOperations.UpdateReservation(server, network, res);
        }

        /// <summary>
        /// Delete a DHCP IP reservation on server
        /// </summary>
        /// <param name="server">server IP address</param>
        /// <param name="network">Subnet network IP address to delete reservation from</param>
        /// <param name="resIp">Reservation IP address to delete</param>
        /// <remarks>Lease associated with reservation will also be deleted</remarks>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning reservation's IP address</permission>
        [WebMethod(Description = "Delete a DHCP IP reservation on server")]
        public void DeleteReservation(DhcpIpAddress server, DhcpIpAddress network, DhcpIpAddress resIp)
        {
            AzServerOperations.DeleteReservation(server, network, resIp);
        }

        /// <summary>
        /// Enumerate IP Reservations owned by subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to enumerate</param>
        /// <returns>An array of reservations on subnet</returns>
        /// <remarks>This method implements security trimming.  User will only 'see' reservations their authorized to access</remarks>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning reservation's IP address</permission>
        [WebMethod(Description = "Enumerate IP Reservations owned by subnet on server")]
        public DhcpReservation[] EnumReservations(DhcpIpAddress server, DhcpIpAddress network)
        {
            return AzServerOperations.EnumReservations(server, network);
        }

        #endregion

        #region Option Ops...
        /// <summary>
        /// Set a DHCP option's value on a particular scope
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="value">DHCP option value</param>
        /// <remarks>This method overwrites any previous data set on value.OptionId</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least Manager role on scope owning option value</permission>
        [WebMethod(Description = "Set a DHCP option's value based on a particular scope")]
        [XmlInclude(typeof(DhcpOptionDataByte)), XmlInclude(typeof(DhcpOptionDataWord)), XmlInclude(typeof(DhcpOptionDataDword)), 
        XmlInclude(typeof(DhcpOptionDataDwordDword)), XmlInclude(typeof(DhcpOptionDataIp)), XmlInclude(typeof(DhcpOptionDataString))]
        public void SetOptionValue(DhcpIpAddress server, DhcpOptionValue value)
        {
            AzServerOperations.SetOptionValue(server, value);
        }

        /// <summary>
        /// Remove a DHCP option's value from a particular scope
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="value">DHCP option value</param>
        /// <remarks>Completes remove all values from option.  Option becomes unset</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least Manager role on scope owning option value</permission>
        [WebMethod(Description = "Remove a DHCP option's value from a particular scope")]
        public void RemoveOptionValue(DhcpIpAddress server, DhcpOptionValue value)
        {
            AzServerOperations.RemoveOptionValue(server, value);
        }

        /// <summary>
        /// Retrieve a DHCP option's value from a particular scope
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="optionId">DHCP option identifier (ex. 006 = DNS Servers)</param>
        /// <param name="classType">DHCP option class type (Dhcp, Bootp, or Routing)</param>
        /// <param name="scope">Get option value from scope (Server, Subnet, or Reservation)</param>
        /// <returns>A DHCP option value</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least Auditor role on scope owning option value</permission>
        [WebMethod(Description = "Retrieve a DHCP option's value from a particular scope")]
        public DhcpOptionValue GetOptionValue(DhcpIpAddress server, UInt32 optionId, DhcpOptionClassType classType, DhcpOptionScope scope)
        {
            return AzServerOperations.GetOptionValue(server, optionId, classType, scope);
        }

        /// <summary>
        /// Enumerate all DHCP option values set in particular scope
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="classType">DHCP option class type (Dhcp, Bootp, or Routing</param>
        /// <param name="scope">Option scope (Server, Subnet, or Reservation) to enumerate</param>
        /// <returns>An array of DHCP option's values current set in scope</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least Auditor role on scope owning option values</permission>
        [WebMethod(Description = "Enumerate all DHCP option values set in particular scope")]
        public DhcpOptionValue[] EnumOptionValues(DhcpIpAddress server, DhcpOptionClassType classType, DhcpOptionScope scope)
        {
            return AzServerOperations.EnumOptionValues(server, classType, scope);
        }

        /// <summary>
        /// Enumerate all DHCP options available on server (global set of DHCP options)
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="classType">DHCP option class type (Dhcp, Bootp, or Routing)</param>
        /// <returns>An array of global DHCP options available on server</returns>
        /// <remarks>Only Dhcp class types seems to return a set of possible DHCP options.  The other classes share these same DHCP options</remarks>
        /// <permission>Does not require authorization, only authentication</permission>
        [WebMethod(Description = "Enumerate all DHCP options available on server (global set of DHCP options)")]
        public DhcpOption[] EnumOptions(DhcpIpAddress server, DhcpOptionClassType classType)
        {
            return AzServerOperations.EnumOptions(server, classType);
        }
        #endregion

    }
}
