/* 
 * DHCP Server Management Objects
 *
 *  File: DhcpSecurity.asmx.cs
 *  Namespace: Unf.Dhcp.Smo.WebServices
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.ComponentModel;
using System.Xml.Serialization;
using Microsoft.Interop.Security.AzRoles;

namespace Unf.Dhcp.Smo.WebServices
{
    /// <summary>
    /// DHCP Web Services Authorization Services
    /// </summary>
    [WebService(Namespace = "http://www.unf.edu/~jrupard/dws/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    public class DhcpSecurity : System.Web.Services.WebService
    {

        #region Perms...

        /// <summary>
        /// Add account with role to DHCP authorization database
        /// </summary>
        /// <param name="scope"></param>
        /// <param name="account"></param>
        [WebMethod(Description = "Add account with role to DHCP authorization database")]
        [XmlInclude(typeof(DhcpAzServerScope)), XmlInclude(typeof(DhcpAzSubnetScope)), XmlInclude(typeof(DhcpAzIpRangeScope))]
        public void AddPermission(DhcpAzScope scope, DhcpAzAccount account)
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);
            DhcpSecurityOps.AddPermission(scope, account.Role, account.AccountName);
        }

        /// <summary>
        /// Remove account with role from DHCP authorization database
        /// </summary>
        /// <param name="scope"></param>
        /// <param name="account"></param>
        [WebMethod(Description = "Remove account with role from DHCP authorization database")]
        public void RemovePermission(DhcpAzScope scope, DhcpAzAccount account)
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);
            DhcpSecurityOps.RemovePermission(scope, account.Role, account.AccountName);
        }

        /// <summary>
        /// Enumerate accounts and their roles in DHCP authorization database
        /// </summary>
        /// <param name="scope"></param>
        /// <returns></returns>
        [WebMethod(Description = "Enumerate accounts and their roles in DHCP authorization database")]
        public DhcpAzAccount[] GetPermissions(DhcpAzScope scope)
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);
            return DhcpSecurityOps.GetPermissions(scope);
        }

        /// <summary>
        /// Add account with global role to DHCP authorization database
        /// </summary>
        /// <param name="account"></param>
        [WebMethod(Description = "Add account with global role to DHCP authorization database")]
        public void AddGlobalPermission(DhcpAzGlobalAccount account)
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);
            DhcpSecurityOps.AddGlobalPermission(account.GlobalRole, account.AccountName);
        }

        /// <summary>
        /// Enumerate global accounts and their roles in DHCP authorization database
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Enumerate global accounts and their roles in DHCP authorization database")]
        public DhcpAzGlobalAccount[] GetGlobalPermissions()
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);
            return DhcpSecurityOps.GetGlobalPermissions();
        }

        /// <summary>
        /// Remove account with global role from DHCP authorization database
        /// </summary>
        /// <param name="account"></param>
        [WebMethod(Description = "Remove account with global role from DHCP authorization database")]
        public void RemoveGlobalPermission(DhcpAzGlobalAccount account)
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);

            DhcpSecurityOps.RemoveGlobalPermission(account.GlobalRole, account.AccountName);
        }

        /// <summary>
        /// Enumerate authorization IP ranges in DHCP authorization database
        /// </summary>
        /// <param name="scope"></param>
        /// <returns></returns>
        [WebMethod(Description = "Enumerate authorization IP ranges in DHCP authorization database")]
        public DhcpAzIpRangeScope[] EnumAzIpRanges(DhcpAzSubnetScope scope)
        {
            if (!DhcpSecurityOps.isAdmin())
                throw new UnauthorizedAccessException(AdminAccessErr);

            return DhcpSecurityOps.EnumAzIpRanges(scope);
        }

        /// <summary>
        /// Check if current user belongs to global administrator role in DHCP authorization database
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Check if current user belongs to global administrator role in DHCP authorization database")]
        public Boolean isAdmin()
        {
            return DhcpSecurityOps.isAdmin();
        }
        #endregion

        #region Const...
        private const String AdminAccessErr = "Must be in the Global Administrator Role or BUILTIN\\Administrators Local Server Group";
        #endregion
    }
}
