/* 
 * DHCP Server Management Objects
 *
 *  File: DhcpSecurityOps.cs
 *  Namespace: Unf.Dhcp.Smo.WebServices
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Configuration;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Text.RegularExpressions;

using System.Runtime.InteropServices;
using Microsoft.Interop.Security.AzRoles;
using System.Security.Principal;

namespace Unf.Dhcp.Smo.WebServices
{
    /// <summary>
    /// Provides a set of security operations associated with Unf.Dhcp.Smo.WebServices.  These operations provide authorization to 
    /// DHCP Web Services using Microsoft's Authorization Manager (AzMan) framework.  Authentication is performed at the web server
    /// level and this application uses the provided user identity (HttpContext.Current.User.Identity) to check authorizations.
    /// </summary>
    /// <seealso href="http://msdn2.microsoft.com/en-us/library/aa480244.aspx">Developing Applications Using Windows Authorization Manager</seealso>
    /// <seealso href="http://technet2.microsoft.com/WindowsServer/en/library/72b55950-86cc-4c7f-8fbf-3063276cd0b61033.mspx?mfr=true">Role-Based Access Control for Multi-tier Applications Using Authorization Manager</seealso>
    /// <seealso href="http://msdn2.microsoft.com/en-us/library/aa375751(VS.85).aspx">Authorization Manager Interfaces</seealso>
    public class DhcpSecurityOps
    {
        /// <summary>
        /// Initializes the AzMan database and application store during application start-up.  It uses AZ_STORE_PATH and AZ_APP_NAME 
        /// from the web.config as parameters for initialization.
        /// </summary>
        /// <returns></returns>
        public static void Init()
        {
            IAzApplication2 AzManApp;
            AzAuthorizationStoreClass AzManStore;
            AzManStore = new AzAuthorizationStoreClass();


            AzManStore.Initialize(0, ConfigurationManager.AppSettings["AZ_STORE_PATH"], null);
            AzManApp = AzManStore.OpenApplication2(ConfigurationManager.AppSettings["AZ_APP_NAME"], null);

            HttpContext.Current.Application["AZ_STORE_OBJ"] = AzManStore;
            HttpContext.Current.Application["AZ_APP_OBJ"] = AzManApp;

        }

        /// <summary>
        /// Gets IAzApplication2 AzMan interface to application data
        /// </summary>
        /// <returns>IAzApplication2 which points to AzMan authorization data for DWS</returns>
        private static IAzApplication2 GetAzApplication()
        {
            return (IAzApplication2)HttpContext.Current.Application["AZ_APP_OBJ"];
        }

        /// <summary>
        /// Gets a AzMan client context based on current user (HttpContext.Current.User.Identity)
        /// </summary>
        /// <returns>Current client context used to check authorized access</returns>
        public static IAzClientContext2 GetClientContext()
        {
            WindowsIdentity wid = (WindowsIdentity)HttpContext.Current.User.Identity;
            IntPtr tokenPtr = wid.Token;
            return (IAzClientContext2)GetAzApplication().InitializeClientContextFromToken((ulong)tokenPtr, 0);
        }

        /// <summary>
        /// Check if current user is a part of builtin\administrators or belongs to the GlobalAdmin role in the AzMan database
        /// </summary>
        /// <returns>True is user is a application admin, otherwise false</returns>
        /// <remarks>builtin\administrators are consider application admins as a fail-safe</remarks>
        public static Boolean isAdmin()
        {
            return HttpContext.Current.User.IsInRole(@"BUILTIN\Administrators") || isAzGlobalAdmin();
        }

        /// <summary>
        /// Check if user in global administrator application role in AzMan database
        /// </summary>
        /// <returns>True is global admin, false otherwise</returns>
        private static Boolean isAzGlobalAdmin()
        {
            IAzClientContext2 ctx = GetClientContext();
            Object[] objRoles = (Object[])ctx.GetRoles("");
            String GlobalAdminRole = Enum.GetName(typeof(DhcpAzGlobalRole), DhcpAzGlobalRole.GlobalAdministrator);

            foreach (String role in objRoles)
                if (String.Compare(GlobalAdminRole, role) == 0)
                    return true;

            return false;
        }

        /// <summary>
        /// Gets an existing AzMan scope object based on scope string
        /// </summary>
        /// <param name="scope">Scope ID string</param>
        /// <returns>A IAzScope object if scope exists, otherwise returns null</returns>
        private static IAzScope GetAzScope(String scope)
        {
            try
            {
                return GetAzApplication().OpenScope(scope, null);
            }
            catch
            {
                return null;
            }
        }

        /// <summary>
        /// Creates AzMan application scope and assigns roles definitions to that scope
        /// </summary>
        /// <param name="scope">Scope ID to be created</param>
        /// <returns>The scope object created</returns>
        /// <remarks>Creates default roles for new scope based on <see cref="DhcpAzRole"/></remarks>
        private static IAzScope CreateCustomAzScope(String scope)
        {
            IAzApplication2 azApp = GetAzApplication();
            IAzScope azScope = azApp.CreateScope(scope, null);
            azScope.Submit(0, null);

            IAzRole role = null;
            foreach (String item in Enum.GetNames(typeof(DhcpAzRole)))
            {
                try
                {
                    //create role in scope
                    role = azScope.CreateRole(item, null);
                    //assign global role definition (role defs are task objects
                    role.AddTask(item, null);
                    role.Submit(0, null);
                }
                finally
                {
                    FreeCom(role, false);
                }
            }

            //create special 'Limited' role
            try
            {
                
                role = azScope.CreateRole("Limited", null);
                //assign global role definition (role defs are task objects
                role.AddTask("Limited", null);
                role.Submit(0, null);
            }
            finally
            {
                FreeCom(role, false);
            }

            return azScope;
        }

        /// <summary>
        /// Enumerate all authorization IP ranges in AzMan database within a subnet
        /// </summary>
        /// <param name="azSubnetScope">Authorization subnet scope to enumerate</param>
        /// <returns>All IP range authorization scopes with subnet scope</returns>
        public static DhcpAzIpRangeScope[] EnumAzIpRanges(DhcpAzSubnetScope azSubnetScope)
        {
            List<DhcpAzIpRangeScope> ranges = new List<DhcpAzIpRangeScope>();
            IAzScopes scopes = GetAzApplication().Scopes;
            String strSubnetScope = azSubnetScope.GenerateAzScopeName();
            Regex filter = new Regex(@"^.*\s-\sIPR\s(\d.*):(\d.*)$");
            foreach (IAzScope scope in scopes)
            {
                try
                {
                    if (scope.Name.Contains(strSubnetScope + " - IPR"))
                    {
                        DhcpAzIpRangeScope ipr = new DhcpAzIpRangeScope();
                        ipr.Server = azSubnetScope.Server;
                        ipr.Subnet = azSubnetScope.Subnet;

                        Match m = filter.Match(scope.Name);
                        ipr.StartIp = new DhcpIpAddress(m.Groups[1].Value);
                        ipr.EndIp = new DhcpIpAddress(m.Groups[2].Value.Trim());
                        ranges.Add(ipr);
                    }
                }
                finally { FreeCom(scope, false); }
            }
            return ranges.ToArray();
        }

        private static object _Lock = new object();
        //private static void UpdateAzCache()
        //{
        //    lock (_Lock)
        //    {
        //        ((AzAuthorizationStoreClass)HttpContext.Current.Application["AZ_STORE_OBJ"]).UpdateCache(null);
        //    }
        //}

        /// <summary>
        /// Gets accounts authorized in scope.  These account belong to at least one role within that scope.
        /// </summary>
        /// <param name="scope">Authorization scope to retrive accounts from</param>
        /// <returns>An array of user or group account belong to authorization scope</returns>
        /// <remarks>Possible scope types: server, subnet, or ipRange</remarks>
        public static DhcpAzAccount[] GetPermissions(DhcpAzScope scope)
        {
            IAzScope azScope = null;
            List<DhcpAzAccount> accounts = new List<DhcpAzAccount>();
            try
            {

                //get scope based on name, null if does not exist
                azScope = DhcpSecurityOps.GetAzScope(scope.GenerateAzScopeName());

                //if scope doesn't exist, create it
                if (azScope == null)
                    return null;

                foreach (IAzRole azRole in azScope.Roles)
                {
                    if (azRole.Name == "Limited")
                        continue;

                    try
                    {
                        foreach (String accountName in ((Object[])azRole.MembersName))
                            accounts.Add(new DhcpAzAccount(accountName, (DhcpAzRole)Enum.Parse(typeof(DhcpAzRole), azRole.Name, true)));
                    }
                    finally { FreeCom(azRole, false); }
                }

            }
            finally
            {
                FreeCom(azScope, false);
            }

            if (accounts.Count == 0)
                return null;
            return accounts.ToArray();
        }

        /// <summary>
        /// Add authorization to AzMan database
        /// </summary>
        /// <param name="scope">Authorization scope to assign account in (server, subnet, ipRange)</param>
        /// <param name="role">Role within scope to assign account</param>
        /// <param name="account">User or group account to authorize</param>
        /// <remarks>Assigns account to limited role at subnet level if scope is at ipRange</remarks>
        public static void AddPermission(DhcpAzScope scope, DhcpAzRole role, String account)
        {

            //make limited role at subnet level for a new permission at ip level
            if (scope.ScopeType == DhcpAzScopeType.IpRange)
            {
                DhcpAzIpRangeScope azIpr = (DhcpAzIpRangeScope)scope;
                DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                azSubnet.Server = azIpr.Server;
                azSubnet.Subnet = azIpr.Subnet;
                AddPermission(azSubnet, "Limited", account);
            }

            String strRole = Enum.GetName(typeof(DhcpAzRole), role);
            AddPermission(scope, strRole, account);
        }

        /// <summary>
        /// Add authorization to AzMan database, internally called by public AddPermission
        /// </summary>
        /// <param name="scope">Authorization scope to assign account in (server, subnet, ipRange)</param>
        /// <param name="role">Role within scope to assign account, see <see cref="DhcpAzRole"/></param>
        /// <param name="account">User or group account to authorize</param>
        private static void AddPermission(DhcpAzScope scope, String role, String account)
        {

            IAzScope azScope = null;
            try
            {
                lock (_Lock)
                {

                    //does scope exist, if not create (check to see if real?)
                    String scopeId = scope.GenerateAzScopeName();

                    //get scope based on name, null if does not exist
                    azScope = DhcpSecurityOps.GetAzScope(scopeId);

                    //if scope doesn't exist, create it
                    if (azScope == null)
                        azScope = DhcpSecurityOps.CreateCustomAzScope(scopeId);

                    IAzRole azRole = null;
                    try
                    {
                        //add account to role, if account already exists, COMException is throw ErrorCode = 0x800700B7
                        azRole = azScope.OpenRole(role, null);
                        azRole.AddMemberName(account, null);
                        azRole.Submit(0, null);
                    }
                    catch (COMException ex)
                    {
                        //ERROR, User already exists in Role
                        if ((uint)ex.ErrorCode != 0x800700B7)
                            throw;

                    }
                    finally
                    {
                        FreeCom(azRole, false);
                    }

                }
            }
            finally
            {
                FreeCom(azScope, false);

            }
        }

        /// <summary>
        /// Remove authorization from AzMan database
        /// </summary>
        /// <param name="scope">Authorization scope to remove account from (server, subnet, ipRange)</param>
        /// <param name="role">Role within scope to remove account from</param>
        /// <param name="account">User or group account to remove authorization</param>
        /// <remarks>Removes account from limited role at subnet level if scope is at ipRange</remarks>
        public static void RemovePermission(DhcpAzScope scope, DhcpAzRole role, String account)
        {
            IAzScope azScope = null;
            Boolean forceGC = false;
            try
            {
                lock (_Lock)
                {
                    
                    //Remove user or group from scope
                    String scopeId = scope.GenerateAzScopeName();
                    azScope = DhcpSecurityOps.GetAzScope(scopeId);

                    if (azScope == null)
                        return;

                    IAzRole azRole = null;
                    try
                    {
                        azRole = azScope.OpenRole(Enum.GetName(typeof(DhcpAzRole), role), null);
                        azRole.DeleteMemberName(account, null);
                        azRole.Submit(0, null);
                    }
                    catch (COMException ex)
                    {
                        //ERROR, element not found
                        if ((uint)ex.ErrorCode != 0x80070490)
                            throw;
                        
                    }
                    finally
                    {
                            FreeCom(azRole, false);
                    }

                    //if scope roles now empty, remove scope
                    if (areRolesEmpty(azScope))
                    {
                        forceGC = true;
                        DeleteCustomAzScope(scopeId);
                    }
                }

            }
            finally
            {
                FreeCom(azScope, forceGC);  
            }

            //make limited role at subnet level for a new permission at ip level
            if (scope.ScopeType == DhcpAzScopeType.IpRange)
            {
                DhcpAzIpRangeScope azIpr = (DhcpAzIpRangeScope)scope;
                DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                azSubnet.Server = azIpr.Server;
                azSubnet.Subnet = azIpr.Subnet;
                RemovePermission(azSubnet, "Limited", account);
            }
        }

        /// <summary>
        /// Remove authorization from AzMan database, internally called by public RemovePermission
        /// </summary>
        /// <param name="scope">Authorization scope to remove account from (server, subnet, ipRange)</param>
        /// <param name="role">Role within scope to remove account from</param>
        /// <param name="account">User or group account to remove authorization</param>
        private static void RemovePermission(DhcpAzScope scope, String role, String account)
        {
            IAzScope azScope = null;
            Boolean forceGC = false;
            try
            {
                lock (_Lock)
                {

                    //Remove user or group from scope
                    String scopeId = scope.GenerateAzScopeName();
                    azScope = DhcpSecurityOps.GetAzScope(scopeId);

                    if (azScope == null)
                        return;

                    IAzRole azRole = null;
                    try
                    {
                        azRole = azScope.OpenRole(role, null);
                        azRole.DeleteMemberName(account, null);
                        azRole.Submit(0, null);
                    }
                    catch (COMException ex)
                    {
                        //ERROR, element not found
                        if ((uint)ex.ErrorCode != 0x80070490)
                            throw;

                    }
                    finally
                    {
                        FreeCom(azRole, false);
                    }

                    //if scope roles now empty, remove scope
                    if (areRolesEmpty(azScope))
                    {
                        forceGC = true;
                        DeleteCustomAzScope(scopeId);
                    }
                }

            }
            finally
            {
                FreeCom(azScope, forceGC);
            }
        }

        /// <summary>
        /// Remove subnet authorization scope and it child scopes in AzMan database
        /// </summary>
        /// <param name="azSubnetScope">Subnet authorization scope to delete and its children</param>
        public static void RemoveAzSubnetAzIpRanges(DhcpAzSubnetScope azSubnetScope)
        {
            DhcpAzIpRangeScope[] childrenScopes = EnumAzIpRanges(azSubnetScope);

            lock (_Lock)
            {
                //delete subnet az
                DeleteCustomAzScope(azSubnetScope.GenerateAzScopeName());

                //delete child ipr az
                foreach (DhcpAzIpRangeScope scope in childrenScopes)
                    DeleteCustomAzScope(scope.GenerateAzScopeName());
            }
        }

        /// <summary>
        /// Add global scope authorization to AzMan database
        /// </summary>
        /// <param name="globalRole">Global Role within global scope to assign account</param>
        /// <param name="account">User or group account to authorize</param>
        public static void AddGlobalPermission(DhcpAzGlobalRole globalRole, String account)
        {
           
            lock (_Lock)
            {

                IAzRole azRole = null;
                try
                {
                    //add account to role, if account already exists, COMException is throw ErrorCode = 0x800700B7
                    //Add members to roles in scope
                    azRole = GetAzApplication().OpenRole(Enum.GetName(typeof(DhcpAzGlobalRole), globalRole), null);
                    azRole.AddMemberName(account, null);
                    azRole.Submit(0, null);
                }
                catch (COMException ex)
                {
                    //ERROR, User already exists in Role
                    if ((uint)ex.ErrorCode != 0x800700B7)
                        throw;

                }
                finally
                {
                    FreeCom(azRole, false);
                }
            } 
            
        }

        /// <summary>
        /// Gets accounts authorized in the global scope.  These account belong to at least one role within the global scope.
        /// </summary>
        /// <returns>An array of user or group account belonging the global authorization scope</returns>
        public static DhcpAzGlobalAccount[] GetGlobalPermissions()
        {
            
            List<DhcpAzGlobalAccount> accounts = new List<DhcpAzGlobalAccount>();

            IAzApplication2 azApp = GetAzApplication();
            foreach (IAzRole azRole in azApp.Roles)
            {
                try
                {
                    foreach (String accountName in ((Object[])azRole.MembersName))
                        accounts.Add(new DhcpAzGlobalAccount(accountName, (DhcpAzGlobalRole)Enum.Parse(typeof(DhcpAzGlobalRole), azRole.Name, true)));
                }
                finally { FreeCom(azRole, false); }
            }

            if (accounts.Count == 0)
                return null;
            return accounts.ToArray();
        }

        /// <summary>
        /// Removes global scope authorization from AzMan database
        /// </summary>
        /// <param name="globalRole">Global Role within global scope to remove account</param>
        /// <param name="account">User or group account to remove authorization</param>
        public static void RemoveGlobalPermission(DhcpAzGlobalRole globalRole, String account)
        {
            lock (_Lock)
            {
                IAzRole azRole = null;
                try
                {
                    azRole = GetAzApplication().OpenRole(Enum.GetName(typeof(DhcpAzGlobalRole), globalRole), null);
                    azRole.DeleteMemberName(account, null);
                    azRole.Submit(0, null);
                }
                catch (COMException ex)
                {
                    //ERROR, element not found
                    if ((uint)ex.ErrorCode != 0x80070490)
                        throw;

                }
                finally
                {
                    FreeCom(azRole, false);
                }
            }
        }
        
        /// <summary>
        /// Delete a AzMan scope created by this appliction
        /// </summary>
        /// <param name="scope">AzMan scope to be deleted</param>
        private static void DeleteCustomAzScope(String scope)
        {
            IAzApplication2 azApp = GetAzApplication();
            try
            {
                azApp.DeleteScope(scope, null);
                azApp.Submit(0, null);
            }
            catch { }
        }

        /// <summary>
        /// Used to determine if a scope's roles are empty of accounts.
        /// </summary>
        /// <param name="azScope">AzMan scope object to query</param>
        /// <returns>True is all roles in scope are empty, otherwise false</returns>
        private static Boolean areRolesEmpty(IAzScope azScope)
        {
            Boolean flag = true;
            foreach (IAzRole azRole in azScope.Roles)
            {
                if (((Object[])azRole.MembersName).Length != 0)
                    flag = false;
                FreeCom(azRole, false);

                if (!flag)
                    break;
            }

            return flag;
        }

        /// <summary>
        /// Free COM object create by AzMan calls
        /// </summary>
        /// <param name="o">COM object to free</param>
        /// <param name="forceGCwait">Force call to on wait garbage collector call (this is a blocking operation)</param>
        private static void FreeCom(object o, bool forceGCwait)
        {
            try
            {
                Marshal.FinalReleaseComObject(o);
            }
            catch { }
            finally 
            { 
                o = null;
                if (forceGCwait)
                {
                    GC.Collect();
                    GC.WaitForPendingFinalizers();
                }
            }
        }

        //test code
        //public static DhcpAzIpRangeScope[] CheckAccessTest()
        //{
        //    //IAzClientContext2 ctx = GetClientContext();

        //    ////scope not found 0x8007013E
        //    //Object[] azScopes = new Object[] { "a" };
        //    //Object[] azOps = new Object[] { DhcpAzOps.Read };
        //    //Object[] results = (Object[])ctx.AccessCheck("CheckTest",azScopes, azOps, null, null, null, null, null);

        //    //foreach (int code in results)
        //    //    if (code != 0)
        //    //        return false;


        //    DhcpAzSubnetScope foo = new DhcpAzSubnetScope();
        //    foo.Server = new DhcpIpAddress("139.62.192.58");
        //    foo.Subnet = new DhcpIpAddress("192.168.45.0");
        //    return EnumAzIpRanges(foo);
        //}

    }

    /// <summary>
    /// AzMan authorization operations
    /// </summary>
    public enum DhcpAzOps
    {
        /// <summary>
        /// Traverse item
        /// </summary>
        Traverse = 1,
        /// <summary>
        /// Read item
        /// </summary>
        Read = 2,
        /// <summary>
        /// Create item
        /// </summary>
        Create = 4,
        /// <summary>
        /// Update item
        /// </summary>
        Update = 8,
        /// <summary>
        /// Delete item
        /// </summary>
        Delete = 16
    }
}
